var test = require('tape');
var FirebaseTemplate = require('..');
var $ = require('jquery');

test('can be instantiated', function (t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';
  document.body.innerHTML = '';

  var template = new FirebaseTemplate();

  t.ok(template instanceof FirebaseTemplate);
  t.end();
});

test('updates dom for simple text input', function (t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';

  document.body.innerHTML = '<input data-path="a" />';

  var template = new FirebaseTemplate();
  template._updateDom({a: 'hello'});

  var html = extractRenderedHTML();

  t.ok(html.match(/hello/g));
  t.end();
});

test('repeats for collections', function (t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';
  document.body.innerHTML = '<div data-path="t" data-type="collection"><span class="repeated"></span></div>';

  var template = new FirebaseTemplate();
  template._updateDom({
    t: {
      a: {created: 1, label: 'A'},
      b: {created: 2, label: 'B'},
      c: {created: 3, label: 'C'}
    }
  });

  t.equal($('.repeated').length, 3);
  t.end();
});

test('collections are ordered by weight', function (t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';
  document.body.innerHTML = '<div data-path="t" data-type="collection"><input data-path="val"></span></div>';

  var template = new FirebaseTemplate();
  template._updateDom({
    t: {
      a: {created: 1, val: 'A'},
      b: {created: 2, val: 'B'},
      c: {created: 3, val: 'C', weight: 0}
    }
  });
  var values = [];
  $("input").each(function() {
    values.push($(this).val());
  });

  t.deepEqual(values, ['C', 'A', 'B']);
  t.end();
});

test('emits content with a data-pathed span', function(t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';
  document.body.innerHTML = '<span data-path="t"/>';

  var template = new FirebaseTemplate();
  template._updateDom({
    t: 'testing'
  });

  t.equal($("span").text(), 'testing');
  t.end();
});

test('supports transforming data when it is updated by an option', function(t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';
  document.body.innerHTML = '<span data-path="t"/>';

  var template = new FirebaseTemplate({transform: function(data) {
    return {t: 'testing'};
  }});
  template._updateDom({});

  t.equal($("span").text(), 'testing');
  t.end();
});

test('supports <a data-href>', function(t) {
  document.body.dataset.path = 'https://fb-tmpl-test.firebaseio.com/';
  document.body.innerHTML = '<a data-href="t" />';

  var template = new FirebaseTemplate();
  template._updateDom({t: 'http://www.testing.com/'});

  t.equal($("a").attr('href'), 'http://www.testing.com/');
  t.end();
});

function extractRenderedHTML() {
  $('input').each(function () {
    var val = $(this).val();
    if (val) {
      $(this).attr('value', val);
    }
  });

  return $('body').html();
}